/*
  Studienprojekt 2002:
  Register-Umsetzung nach XML
  Autor:   Heejung Park 
  E-Mail: toheejung@web.de

*/

import java.io.*;
import java.util.*;
import java.text.*;
import javax.swing.*;

public class Converter extends Thread {

  final String SEE_ALSO = "s. auch";
  final String SEE = "s.";
  final String PUBLISHER = "(Hrsg.)";
  final String LOGFILE = "logfile.txt";

  private TagStack tagStack;
  private MetaTable metaTable;
  private BufferedReader in;
  private XMLDocument xmlDoc;
  
  private JDialog convDialog;
  private JTextArea txtArea;


  public Converter(JDialog convDialog, JTextArea txtArea,
                   File rtfFile, File metaFile, File xmlFile) throws IOException {

    this.convDialog = convDialog;
    this.txtArea = txtArea;

    tagStack = new TagStack();
    metaTable = new MetaTable(metaFile);

//  RTF Datei
    in = new BufferedReader(new FileReader(rtfFile));
//  XML Dokument
    xmlDoc = new XMLDocument(xmlFile, metaTable.getValue("TITLE"));
  }

  public void run() {
  
    BufferedWriter logFile;   // interne Protokolldatei
    String rtfLine;
    int lastTab;
    String tag;
    String delim;
    String leftTag, leftToken;
    String rightTag, rightToken;
    String token, nextToken, lastToken;
    String context;
    String metaKey;
    Vector resultList;  // aus Meta-Tabelle gelesene Tags
    boolean foundStart;

//  Initialisierung
    RTFTokenizer.init();
    lastTab = 0;
    context = "";
    metaKey = "";
    foundStart = false;

    try {
      sleep(2500);

      while ( !isInterrupted() && (rtfLine = in.readLine()) != null ) {

        if (!foundStart) {
//        Startposition im RTF Text ermitteln und evtl. formatieren
          if (!rtfLine.trim().startsWith("\\par "))
            continue;
          else {
            if ( rtfLine.startsWith("\\par }\\pard") )
              if ( rtfLine.indexOf("{") > 0 )
                rtfLine = rtfLine.substring(0, 5) +
                          rtfLine.substring(rtfLine.indexOf("{"), rtfLine.length());
              else
                continue;
            foundStart = true;
          }
        }

//      interne Protokolldatei zur mglichen Fehleranlayse
        logFile = new BufferedWriter(new FileWriter(LOGFILE));
        logFile.write(rtfLine);
        logFile.close();  // nur letzten RTF Satz sichern

        RTFTokenizer rtfTokenizer = new RTFTokenizer(rtfLine);
        token = rtfTokenizer.nextToken();
        if (token != null) {

          if (rtfTokenizer.tab == 0) {
            lastTab = 0;
            tagStack.closeAllTags(xmlDoc);

//          Prfe ob " s. auch " oder "s." im Text!
            delim = "" ;
            if (token.indexOf(SEE_ALSO) > 0 )
              delim = SEE_ALSO;
            else
              if (token.indexOf(SEE) > 0 )
                delim = SEE;

            if (!delim.equals("")) {
//            Text enthlt " s. auch " oder " s. "!
              leftToken = XMLDocument.getLeftString(token, delim);
              metaKey = "";
              if ( rtfTokenizer.font.equals("") && leftToken.indexOf(",") > 0 ) {
                metaKey = "STRING_KOMMA_STRING";
                metaKey = metaKey.concat("SEE");
              }

              if ( rtfTokenizer.font.equals("") && leftToken.indexOf(",") <= 0 ) {
                metaKey = "STRING";
                metaKey = metaKey.concat("SEE");
              }

              rightToken = XMLDocument.getRightString(token, delim);
              metaKey = metaKey.concat("STRING");
              if (rightToken.endsWith(",")) {
                rightToken = rightToken.substring(0, rightToken.length()-1);
                metaKey = metaKey.concat("KOMMA");
                token = rtfTokenizer.nextToken();
                if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) )
                  metaKey = metaKey.concat("STRING_ITALIC");
              }
              resultList = (Vector) metaTable.getTags(metaKey);

              tag = (String) resultList.elementAt(0);
              xmlDoc.writeElement(tag, leftToken, tagStack);

              tag = (String)resultList.elementAt(1);
              xmlDoc.writeElement(tag, rightToken);

              if (resultList.size() > 2) {
                tag = (String)resultList.elementAt(2);
                xmlDoc.writeElement(tag, token);
              }

              if (!tagStack.empty()) {
                tag = (String)tagStack.pop();
                xmlDoc.writeEndTag(tag);
              }
              token = "";
            }

            if (delim.equals("")) {
//       -> Text enthlt kein " s. auch " oder " s. "!

//            Keine Formatierung und kein Komma im Text?
              if ( rtfTokenizer.font.equals("") && token.indexOf(",") < 0 ) {
                resultList = (Vector) metaTable.getTags("STRING");
                tag = (String) resultList.elementAt(0);
                context = xmlDoc.writeElement(tag, token, tagStack, true);
              }

//            Keine Formatierung und Komma im Text?
              if ( rtfTokenizer.font.equals("") && token.indexOf(",") > 0 ) {
                resultList = (Vector) metaTable.getTags("STRING_KOMMA_STRING");
                tag = (String) resultList.elementAt(0);
                context =  xmlDoc.writeElement(tag, token, tagStack, true);
              }

//            Formatierung ITALIC (Komma nicht relevant)?
              if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) ) {
                metaKey = "STRING_ITALIC";
//              Lese zweiten Text, wenn vorhanden!
                nextToken = rtfTokenizer.nextToken();

                if (nextToken != null) {
//                Prfe ob "s. auch " oder "s." im zweiten Text!
                  delim = "" ;
                  if (nextToken.startsWith(SEE_ALSO))
                    delim = SEE_ALSO;
                  else
                    if (nextToken.startsWith(SEE))
                      delim = SEE;

//                Zweiter Text enthlt kein "s. auch" oder "s."?
                  if (delim.equals("")) {
                    resultList = (Vector) metaTable.getTags(metaKey);
                    tag = (String) resultList.elementAt(0);
                    token = token.concat(nextToken);
                    context =  xmlDoc.writeElement(tag, token, tagStack, true);
                  }
                  else {
//                  Zweiter Text enthlt "s. auch" oder "s."!
                    metaKey = metaKey.concat("SEE");
                    if ( rtfTokenizer.font.equals("") ) {
                      metaKey = metaKey.concat("STRING");

                      lastToken = "";
                      if (nextToken.endsWith(",")) {
                        nextToken = nextToken.substring(0, nextToken.length()-1);
                        metaKey = metaKey.concat("KOMMA");
                        lastToken = rtfTokenizer.nextToken();
                        if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) )
                          metaKey = metaKey.concat("STRING_ITALIC");
                      }

                      resultList = (Vector) metaTable.getTags(metaKey);

                      if (resultList != null) {
                        nextToken = XMLDocument.getRightString(nextToken, delim);
                        tag = (String)resultList.elementAt(0);
                        xmlDoc.writeElement(tag, token, tagStack);
                        tag = (String)resultList.elementAt(1);
                        xmlDoc.writeElement(tag, nextToken);
                        if (resultList.size() > 2) {
                          tag = (String)resultList.elementAt(2);
                          xmlDoc.writeElement(tag, lastToken);
                        }
                      }
                    }
                  }
                }
                else {
//                Formatierung ITALIC und kein zweiter Text vorhanden!
                  resultList = (Vector) metaTable.getTags(metaKey);
                  tag = (String) resultList.elementAt(0);
                  context =  xmlDoc.writeElement(tag, token, tagStack, true);
                }
              }
            }

          }

//        Tabulatoren vorhanden?
          if (rtfTokenizer.tab != 0) {

            if (lastTab >= rtfTokenizer.tab) {
              if (lastTab >= tagStack.size()) {
                lastTab = tagStack.size()-1;    // Korrektur von Unregelmigkeiten
              }
              for (int i = 0; i <= (lastTab-rtfTokenizer.tab); i++)
                tagStack.closeLastTag(xmlDoc);
            }
            lastTab = rtfTokenizer.tab;

            metaKey = "";
            for (int i = 0; i < rtfTokenizer.tab; i++) {
              metaKey = metaKey.concat("TAB");
            }

//          Tabulator(en) und Kontext Person oder Sonstiges?
            if ( metaTable.getContext("STRING_KOMMA_STRING").equals(context) ||
                 metaTable.getContext("STRING").equals(context)) {
              metaKey = context.concat(metaKey);

//            Formatierung ITALIC!
              if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) ) {
                metaKey = metaKey.concat("STRING_ITALIC");
                nextToken = rtfTokenizer.nextToken();
                if (nextToken != null) {
                  if ( rtfTokenizer.font.equals("") ) {
                    metaKey = metaKey.concat("_STRING");
                    token = token.concat(" "+nextToken);
                    nextToken = rtfTokenizer.nextToken();
                    if (nextToken != null) {
                      if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) ) {
                        metaKey = metaKey.concat("_STRING_ITALIC");
                        token = token.concat(" "+nextToken);
                      }
                    }
                  }
                }
                resultList = (Vector) metaTable.getTags(metaKey);
                tag = (String) resultList.elementAt(0);
                xmlDoc.writeElement(tag, token, tagStack, true);
              }
              else {
//              Keine Formatierung!
                if ( rtfTokenizer.font.equals("") ) {
//                Beginnt mit "(Hrsg.)"!
                  if (token.startsWith(PUBLISHER)) {
                    metaKey = metaKey.concat("PUBLISHER");
                    int beginIndex = 7;
                    int endIndex = token.length();
                    if (endIndex > beginIndex) {
                      token = token.substring(beginIndex, endIndex);
                      metaKey = metaKey.concat("STRING");
                      if (token.endsWith(","))
                        token = token.substring(0, token.length()-1).trim();
                    }
                    nextToken = rtfTokenizer.nextToken();
                    if (nextToken != null && rtfTokenizer.font.equals(RTFTokenizer.ITALIC)) {
                      metaKey = metaKey.concat("STRING_ITALIC");
                      resultList = (Vector) metaTable.getTags(metaKey);
                      String mainTag = (String)resultList.elementAt(0);
                      xmlDoc.writeStartTag(mainTag);
                      tagStack.addElement(mainTag);

                      if (resultList.size() == 3) {
                        tag = (String)resultList.elementAt(1);
                        xmlDoc.writeElement(tag, token);
                        tag = (String)resultList.elementAt(2);
                        xmlDoc.writeElement(tag, nextToken);
                      }
                      else {
                        tag = (String)resultList.elementAt(1);
                        xmlDoc.writeElement(tag, nextToken);
                      }
                      // xmlDoc.writeEndTag(mainTag);
                    }
                  }
//                (Keine Formatierung!) Beginnt nicht mit "(Hrsg.)"!
                  if (!token.startsWith(PUBLISHER)) {
                    metaKey = metaKey.concat("STRING");
                    nextToken = rtfTokenizer.nextToken();
                    if (nextToken != null && rtfTokenizer.font.equals(RTFTokenizer.ITALIC)) {
                      metaKey = metaKey.concat("_STRING_ITALIC");
                      token = token.concat(" "+nextToken);
                    }
                    resultList = (Vector) metaTable.getTags(metaKey);
                    if (resultList != null) {
                      tag = (String) resultList.elementAt(0);
                      xmlDoc.writeElement(tag, token, tagStack, true);
                    }
                  }
                }
              }
            }

//          Tabulator(en) und Kontext Werk?
            if ( metaTable.getContext("STRING_ITALIC").equals(context) ) {
              metaKey = context.concat(metaKey);

//            Formatierung ITALIC!
              if ( rtfTokenizer.font.equals(RTFTokenizer.ITALIC) ) {
                metaKey = metaKey.concat("STRING_ITALIC");
                resultList = (Vector) metaTable.getTags(metaKey);
                if (resultList != null) {
                  tag = (String) resultList.elementAt(0);
                  xmlDoc.writeElement(tag, token, tagStack);
                }
              }
              else {
//              Keine Formatierung ITALIC!
                if ( rtfTokenizer.font.equals("") ) {
                  metaKey = metaKey.concat("STRING");
                  resultList = (Vector) metaTable.getTags(metaKey);
                  if (resultList != null) {
                    tag = (String) resultList.elementAt(0);
                    xmlDoc.writeElement(tag, token, tagStack, true);
                  }
                }
              }

            }

          }
//        Ermittlung der letzten offenen Formatierung in einer Zeile!
          while (rtfTokenizer.nextToken() != null) {}
        }
      } // while (!isInterrupted ...
      tagStack.closeAllTags(xmlDoc);

      xmlDoc.close();
      in.close();

//    Textbereich (Protokoll) fortschreiben
      Date now = new Date();
      DateFormat formatdt = DateFormat.getDateTimeInstance();
      txtArea.append(System.getProperty("line.separator")+formatdt.format(now)+"End Conversion");

//    Konvertierungsdialog ausblenden
      if (convDialog != null)
        convDialog.setVisible(false);
    }
    catch (InterruptedException ex) {
      isInterrupted();
      txtArea.append("Internal Error: Please restart program!");
      ex.printStackTrace();
    }
    catch (IOException ex) {
      txtArea.append("Conversion Error: Please check input files!");
			ex.printStackTrace();
    }
  }

}




