/*
  Studienprojekt 2002:
  Register-Umsetzung nach XML
  Autor:   Heejung Park
  E-Mail: toheejung@web.de
*/

/*
Die Klasse MetaTable ist von der Klasse Hashtable abgeleitet. 
Das heisst, die Klasse MetaTable erbt alle Eigenschaften (Methoden und Attribute) 
der Klasse Hashtable. Beim Aufruf von Metatable kann man also alle Methodedn von Hashtable nutzen.
Im Rahmen der Vererbung knnte man diese Methoden von Hashtable auch in der Klasse MetaTable
ueberschreiben. 
Die Methode "fillMetaTable" sucht ";"(Semikolon) von ausgewhlten Metadateien, 
die als CSV(Comma Separated Value) gespeichert ist, und baut Metatabelle auf. 

Hashtable: eine Hashtabelle speichert zu einem Schluessel (Key) einen Wert (Value) und 
dient zum schnellen Wiederauffinden eines Wertes anhand eines Schlssels. 
Der interne Index eines Eintrages wird aus dem int hashCode() des Schluessels errechnet. 
Im Eintrag werden Schluessel und Wert gespeichert. 

Iterator (Schleife) erzeugt die Filterliste:
die Methode sucht im HAshtabelle die Wert "&", nimmt die Werte, die vor "&" stehen
und addiert die Werte in einer Filterliste. Wenn die gleiche Werte mehr als zweimal vorkommen, 
werden die doppelten Werte nicht addiert.     

*/
import java.io.*;
import java.util.*;
import java.math.*;

public class MetaTable extends Hashtable {

  private BufferedReader metaFile = null;
  private String metaLine = null;

  public MetaTable(File fileName) throws IOException {
    super(20);
    try {
      metaFile = new BufferedReader(new FileReader(fileName));
      if (metaFile != null) {
        fillMetaTable();
        metaFile.close();
      }  
    }
    catch (FileNotFoundException e){
      System.out.println("File not found: "+fileName);
    }
  }

  private void fillMetaTable() throws IOException {

    if (metaFile != null) metaFile.readLine();

    while ((metaLine = metaFile.readLine())!= null) {
      String key = "", value = "";
      int count = 0;
      boolean isKey = true;
      StringTokenizer stringTokenizer = new StringTokenizer(metaLine, ";",true);

      while (stringTokenizer.hasMoreTokens()) {
        String token = stringTokenizer.nextToken();
        if (token.equals(";"))
          count++;
        else {
          if (isKey)
            key = key.concat(token);
          else
            value = value.concat(token+";");
        }
        if (count == 9) isKey = false;
      }
      if (value.endsWith(";")) value = value.substring(0, value.length()-1);
      put(key, value);
    }
  }

  public String getValue(String key) {
    String value = (String) get(key);
    return value;
  }

  public String getTag(String key) {
    return getValue(key);
  }

  public Vector getTags(String key) {
    Vector tagList = new Vector(5);
    String value = getTag(key);
    if (value != null) {
      StringTokenizer stringTokenizer = new StringTokenizer(value, ";", true);
      while (stringTokenizer.hasMoreTokens()) {
         String token = stringTokenizer.nextToken();
         if (!token.equals(";"))
           tagList.addElement(token);
      }
      return tagList;
    }
    return null;
  }

  public String getContext(String key) {
    String tag = getValue(key);
    String leftTag = XMLDocument.getLeftString(tag, "&");
    if (leftTag != null)
      return leftTag;
    else
      return tag;
  }

}



