/*
  Studienprojekt 2002:
  Register-Umsetzung nach XML
  Autor:   Heejung Park
  E-Mail: toheejung@web.de
*/

/*
Die Klassed RTFTokenizer zerlegt RTF-Dati.
Die Klasse RTFTokenizer ist von "StringTokenizer" abgeleitet.

Die Klasse StringTokenizer ist eine Hilfsklasse, damit 
knnen die Strings in einzelne Tokens zerlegt werden. 
Ein Token wird dabei als zusammenhaengende Sequenz von Zeichen angesehen, 
die durch Trennzeichen oder durch das Ende der Zeichenkette begrenzt ist.  
*/


import java.io.*;
import java.util.*;
import java.math.*;

public class RTFTokenizer extends StringTokenizer {

  static final String ITALIC = "italic";
  static final String BOLD = "bold";
  final String NUMBER_CODE = "01234567890f.-,)( ";

  static Hashtable rtfCodeTable = null;
  static String font = "";
  static int tab = 0;
  static boolean isOpenPard = false;
  private BufferedReader rtfCodeFile = null;
  private String rtfCodeLine = "";

  public RTFTokenizer(String rtfStr) throws IOException {
    super(rtfStr, "{}", true);
    try {
      if (rtfCodeTable == null || rtfCodeTable.isEmpty()) {
        rtfCodeTable = new Hashtable(20);
        rtfCodeFile = new BufferedReader(new FileReader("rtfcode.csv"));
        if (rtfCodeFile != null) {
          fillRTFCodeTable();
          rtfCodeFile.close();
        }
      }
    }
    catch (FileNotFoundException e){
      System.out.println("File not found: "+"rtfcode.csv");
    }
  }

  public String nextToken() {
    boolean foundTab = true;

    while (hasMoreTokens()) {
      String token = super.nextToken();
      int index = -1;

      if (token.equals("{")) continue;

      if (token.equals("}")) {
        font = "";
        continue;
      }

      if (token.startsWith("[")) continue;

      if (token.startsWith("\\i [")) continue;

      if (token.startsWith("\\pard \\s29\\qj\\fi")) {
        tab++ ;
        foundTab = true;
        continue;
      }

      if (token.startsWith("\\pard \\s29\\qj\\li")) {
        tab-- ;
        foundTab = false;
        continue;
      }
      
      if (token.startsWith("\\pard")) {
        if (isOpenPard) {
          isOpenPard = false;
          tab = 0;
        }
        else {
          isOpenPard = true;
          tab = 1;
        }
        continue;
      }

      if (token.startsWith("\\i ")) {
        font = ITALIC;
        if (token.length() >= 3)
          token = token.substring(3, token.length());
        else
          continue;
      }

      if (token.startsWith("\\b ")) {
        font = BOLD;
        if (token.length() >= 3)
          token = token.substring(3, token.length());
        else
          continue;
      }

      if (token.startsWith("\\b\\i")) {
        font = BOLD;                    // future: "bolditalic"
        if (token.length() >= 4)
          token = token.substring(4, token.length());
        else
          continue;
      }

      if (token.startsWith("\\i\\lang")) {
        font = ITALIC;
        if ((index = token.trim().indexOf(" ")) >= 0)
          token = token.substring(index+1, token.length());
        else
          continue;
      }

      if (token.startsWith("\\b\\lang")) {
        font = BOLD;
        index = token.indexOf(" ");
        if ((index = token.trim().indexOf(" ")) >= 0)
          token = token.substring(index+1, token.length());
        else
          continue;
      }

      if (token.startsWith("\\b\\i\\lang")) {
        font = BOLD;                    // future: "bolditalic"
        index = token.indexOf(" ");
        if ((index = token.trim().indexOf(" ")) >= 0)  
          token = token.substring(index+1, token.length());
        else
          continue;
      }

      if (token.startsWith("\\tab ")) {
        foundTab = true;
        if (tab > 0) tab = 1;
        while (token.startsWith("\\tab ")) {
          tab++;
          if (token.length() > 4)
            token = token.substring(5, token.length());
        }
      }

      if (token.startsWith("\\par ")) {
        foundTab = false;
        if (token.length() > 5) {
          token = token.substring(5, token.length());
          if (isOpenPard)
            tab = 1;
          else
            tab = 0;
          while (token.startsWith("\\tab ")) {
            foundTab = true;
            tab++;
            if (token.length() > 4)
              token = token.substring(5, token.length());
          }
          if (token.trim().equals("")) continue;
        }
        else
          continue;
      }

      token = token.trim();
      if (token.startsWith("\\lang")) {
        if ((index = token.indexOf(" ")) >= 0)
          token = token.substring(index+1, token.length());
        else
          continue;
      }

      if (token.length() > 0) {
        if (font.equals(BOLD)) continue;  // future use
        if (token.startsWith("[")) continue;
        if (isNumber(token)) continue;
        if (isOpenPard == false && foundTab == false) tab = 0;
        if (isOpenPard == true && foundTab == false) tab = 1;
        token = replaceRTFCode(token);
        return token;
      }

    }
    if (isOpenPard == false && foundTab == false) tab = 0;
    if (isOpenPard == true && foundTab == false) tab = 1;
    return null;
  }

  private boolean isNumber(String token) {
    String number = NUMBER_CODE;
    for (int i=0; i<token.length(); i++) {
      if (number.indexOf(token.charAt(i)) <  0) return false;
    }
    return true;
  }

  private String replaceRTFCode(String token) {
    String rtfCode = "";
    int pos = 0;

    while((pos = token.indexOf("\\'", pos)) >= 0) {
      rtfCode = token.substring(pos+2, pos+4);
      String asciiCode = (String)rtfCodeTable.get(rtfCode);
      if (asciiCode != null) {
        StringBuffer token_wrk = new StringBuffer(token);
        token_wrk.replace(pos, pos+4, asciiCode);
        token = token_wrk.toString();
      }
      pos++;
    }
    token = token.replace('&', 'u');    // because of XML
    return token;
  }

  private void fillRTFCodeTable() throws IOException {

    while ((rtfCodeLine = rtfCodeFile.readLine())!= null) {
      StringTokenizer st = new StringTokenizer(rtfCodeLine, ";");
      if (st.hasMoreTokens()) {
        String key = st.nextToken();       // RTF code
        if (st.hasMoreTokens()) {
          String value = st.nextToken();   // ASCII code
          rtfCodeTable.put(key, value);
        }
      }
    }
  }

  static void init() {
    font = "";
    isOpenPard = false;
    tab = 0;
    rtfCodeTable = null;
  }

}
