/*
  Studienprojekt 2002:
  Register-Umsetzung nach XML
  Autor:   Heejung Park
  E-Mail: toheejung@web.de
*/

import java.io.*;
import java.util.*;
import java.math.*;

public class Separator {

  private BufferedReader xmlFile;
  private XMLDocument xmlDocument;


  public Separator(boolean separateFirstLevel, String inputFilter,
                   File xmlSepFile, String titleTag) throws IOException {

    xmlFile = new BufferedReader(new FileReader(xmlSepFile));
    if (separateFirstLevel) {
      xmlDocument = new XMLDocument(inputFilter+".xml", inputFilter);
      convertFirstLevel(inputFilter, titleTag);
    }
    else {
      xmlDocument = new XMLDocument(inputFilter+"_.xml", inputFilter+"_");
      convertAllLevels(inputFilter, titleTag);
    }

    xmlFile.close();
    xmlDocument.close();
  }

  private void convertFirstLevel(String inputFilter, String titleTag) throws IOException {
    String xmlLine = null;
    StringTokenizer stringTokenizer;
    String token, skipFilter;
    int openNumber = 0;
    int closeNumber = 0;
    int count = 0;
    boolean isOpenTag = false;


    while ((xmlLine = xmlFile.readLine())!= null) {
      if (xmlLine.trim().startsWith("<"+titleTag+">")) break;
    }

    while ((xmlLine = xmlFile.readLine())!= null) {
      stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
      if (stringTokenizer.hasMoreTokens()) {
        token = stringTokenizer.nextToken();

        if (token.equals(inputFilter)) {
          openNumber++;
          isOpenTag = true;
          if (openNumber == 1) {
            count++;
            int index = xmlLine.indexOf("<"+inputFilter);
            StringBuffer xmlBufferLine = new StringBuffer(xmlLine);
            xmlBufferLine = xmlBufferLine.replace(index+1, index+inputFilter.length()+1, inputFilter+"_"+String.valueOf(count).trim());
            xmlLine = xmlBufferLine.toString();
          }
          xmlDocument.writeLine(xmlLine);

          while ((xmlLine = xmlFile.readLine())!= null) {
            stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
            if (stringTokenizer.hasMoreTokens()) {
              token = stringTokenizer.nextToken();
              if (token.equals(inputFilter)) {
                openNumber++;
                isOpenTag = true;
                if (openNumber == 1) {
                  count++;
                  int index = xmlLine.indexOf("<"+inputFilter);
                  StringBuffer xmlBufferLine = new StringBuffer(xmlLine);
                  xmlBufferLine = xmlBufferLine.replace(index+1, index+inputFilter.length()+1, inputFilter+"_"+String.valueOf(count).trim());
                  xmlLine = xmlBufferLine.toString();
                }
                xmlDocument.writeLine(xmlLine);
              }
              else {
                if (token.equals("/"+inputFilter)) {
                  closeNumber++;
                  if (openNumber == closeNumber) {
                    isOpenTag = false;
                    openNumber = closeNumber = 0;
                    int index = xmlLine.indexOf("</"+inputFilter);
                    StringBuffer xmlBufferLine = new StringBuffer(xmlLine);
                    xmlBufferLine = xmlBufferLine.replace(index+2, index+inputFilter.length()+2, inputFilter+"_"+String.valueOf(count).trim());
                    xmlLine = xmlBufferLine.toString();
                    xmlDocument.writeLine(xmlLine);
                    break;
                  }
                  xmlDocument.writeLine(xmlLine);
                }
                else {
                  if (isOpenTag)
                    xmlDocument.writeLine(xmlLine);
                }
              }
            } // if (stringTokenizer...
          } // while ((xmlLine...
        }
        else {  // if (token.equals(inputFilter))
          skipFilter = token;
          openNumber = closeNumber = 0;
          openNumber++;

          while ((xmlLine = xmlFile.readLine())!= null) {
            stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
            if (stringTokenizer.hasMoreTokens()) {
              token = stringTokenizer.nextToken();
              if (token.equals(skipFilter)) {
                openNumber++;
              }
              else {
                if (token.equals("/"+skipFilter)) {
                  closeNumber++;
                  if (openNumber == closeNumber) {
                    openNumber = closeNumber = 0;
                    break;
                  }
                }
              } // if (token.equals(skipFilter))
            } // if (stringTokenizer
          } // while ((xmlLine...
        } // if (token.equals(inputFilter))
      } // if (stringTokenizer...
    } // while ((xmlLine...
  }

  private void convertAllLevels(String inputFilter, String titleTag) throws IOException {
    String xmlLine = null;
    StringTokenizer stringTokenizer;
    String token, subFilter;
    int openNumber = 0;
    int closeNumber = 0;
    boolean isOpenTag = false;
    boolean isInputFilter = false;
    Vector buffer = new Vector();


    while ((xmlLine = xmlFile.readLine())!= null) {
      if (xmlLine.trim().startsWith("<"+titleTag+">")) break;
    }

    while ((xmlLine = xmlFile.readLine())!= null) {
      stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
      if (stringTokenizer.hasMoreTokens()) {
        token = stringTokenizer.nextToken();

        if (token.equals(inputFilter)) {
          openNumber++;
          isOpenTag = true;
          xmlDocument.writeLine(xmlLine);

          while ((xmlLine = xmlFile.readLine())!= null) {
            stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
            if (stringTokenizer.hasMoreTokens()) {
              token = stringTokenizer.nextToken();
              if (token.equals(inputFilter)) {
                openNumber++;
                isOpenTag = true;
                xmlDocument.writeLine(xmlLine);
              }
              else {
                if (token.equals("/"+inputFilter)) {
                  closeNumber++;
                  if (openNumber == closeNumber) {
                    isOpenTag = false;
                    openNumber = closeNumber = 0;
                    xmlDocument.writeLine(xmlLine);
                    break;
                  }
                  xmlDocument.writeLine(xmlLine);
                }
                else {
                  if (isOpenTag)
                    xmlDocument.writeLine(xmlLine);
                }
              }
            } // if (stringTokenizer...
          } // while ((xmlLine...
        }
        else {  // if (token.equals(inputFilter))
          isInputFilter = false;
          subFilter = token;
          openNumber = closeNumber = 0;
          openNumber++;

          buffer.clear();
          buffer.addElement(xmlLine);

          while ((xmlLine = xmlFile.readLine())!= null) {
            stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
            if (stringTokenizer.hasMoreTokens()) {
              token = stringTokenizer.nextToken();
              buffer.addElement(xmlLine);
              if (token.equals(inputFilter))
                isInputFilter = true;
              else {
                if (token.equals(subFilter))
                  openNumber++;
                else {
                  if (token.equals("/"+subFilter)) {
                    closeNumber++;
                    if (openNumber == closeNumber) {
                      openNumber = closeNumber = 0;
                      if (isInputFilter) {
                        for (int i = 0; i < buffer.size(); i++)
                          xmlDocument.writeLine((String)buffer.elementAt(i));
                        buffer.clear();
                      }
                      break;
                    }
                  }
                }
              } // if (token.equals(inputFilter))
            } // if (stringTokenizer
          } // while ((xmlLine...
        } // if (token.equals(inputFilter))
      } // if (stringTokenizer...
    } // while ((xmlLine...
  }

  private void convertFromLevel(String inputFilter) throws IOException {
    String xmlLine = null;
    StringTokenizer stringTokenizer;
    String token;
    int openNumber = 0;
    int closeNumber = 0;
    int count = 0;
    boolean isOpenTag = false;

    while ((xmlLine = xmlFile.readLine())!= null) {
      stringTokenizer = new StringTokenizer(xmlLine, "< >", false);
      if (stringTokenizer.hasMoreTokens()) {
        token = stringTokenizer.nextToken();

        if (token.equals(inputFilter)) {
          openNumber++;
          isOpenTag = true;
          if (openNumber == 1) {
            count++;
            int index = xmlLine.indexOf("<"+inputFilter);
            StringBuffer xmlBufferLine = new StringBuffer(xmlLine);
            xmlBufferLine = xmlBufferLine.replace(index+1, index+inputFilter.length()+1, inputFilter+"_"+String.valueOf(count).trim());
            xmlLine = xmlBufferLine.toString();
          }
          xmlDocument.writeLine(xmlLine);
        }
        else {
          if (token.equals("/"+inputFilter)) {
            closeNumber++;
            if (openNumber == closeNumber) {
              isOpenTag = false;
              openNumber = closeNumber = 0;
              int index = xmlLine.indexOf("</"+inputFilter);
              StringBuffer xmlBufferLine = new StringBuffer(xmlLine);
              xmlBufferLine = xmlBufferLine.replace(index+2, index+inputFilter.length()+2, inputFilter+"_"+String.valueOf(count).trim());
              xmlLine = xmlBufferLine.toString();
            }
            xmlDocument.writeLine(xmlLine);
          }
          else {
            if (isOpenTag)
              xmlDocument.writeLine(xmlLine);
          }
        }
      } // if (stringTokenizer...
    } // while ((xmlLine...
  }

}






