/*
  Studienprojekt 2002:
  Register-Umsetzung nach XML
  Autor:   Heejung Park
  E-Mail: toheejung@web.de
*/

/*
Die Klasse XMLDocument schreibt XML-Datei.
Die <tags> fuer XML-Datei werden von MetaTable bernommen. 
Hier wurde die Klasse "BufferedWriter" verwendet, 
diese Klasse hat die Aufgabe, Streamausgabe zu puffern. 
*/

import java.io.*;
import java.util.*;
import java.math.*;

public class XMLDocument {

  private BufferedWriter xmlFile = null;
  private String title = null;

  public XMLDocument(String fileName) throws IOException {
    xmlFile = new BufferedWriter(new FileWriter(fileName));
    xmlFile.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>");
    xmlFile.newLine();
  }

  public XMLDocument(String fileName, String documentTitle) throws IOException {
    this(fileName);
    title = documentTitle;
    xmlFile.write("<"+documentTitle+">");
    xmlFile.newLine();
  }

  public XMLDocument(File file) throws IOException {
    xmlFile = new BufferedWriter(new FileWriter(file));
    xmlFile.write("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>");
    xmlFile.newLine();
  }

  public XMLDocument(File file, String documentTitle) throws IOException {
    this(file);
    title = documentTitle;
    xmlFile.write("<"+documentTitle+">");
    xmlFile.newLine();
  }

  public void writeStartTag(String tag) throws IOException {
    xmlFile.write("<"+tag+">");
    xmlFile.newLine();
  }

  public void writeStartTag(String tag, String attribute) throws IOException {
    if (attribute.startsWith("(") && attribute.endsWith(")"))
      attribute = attribute.substring(1, attribute.length()-1).trim();
    xmlFile.write("<"+tag+" misc=\""+attribute+"\">");
    xmlFile.newLine();
  }

  public void writeEndTag(String tag) throws IOException {
    xmlFile.write("</"+tag+">");
    xmlFile.newLine();
  }

  public void writeLine(String XMLLine) throws IOException {
    xmlFile.write(XMLLine);
    xmlFile.newLine();
  }

  public void writeText(String tag, String text) throws IOException {
    xmlFile.write("<"+tag+">"+text+"</"+tag+">");
    xmlFile.newLine();
  }

  public void writeText(String tag, String text, String attribute) throws IOException {
    if (attribute.startsWith("(") && attribute.endsWith(")"))
      attribute = attribute.substring(1, attribute.length()-1).trim();
    xmlFile.write("<"+tag+" misc=\""+attribute+"\">"+text+"</"+tag+">");
    xmlFile.newLine();
  }

  public void writeElement(String tag, String text) throws IOException {
    if (tag.indexOf("&") > 0) {
      String leftTag = getLeftString(tag, "&");
      String rightTag = getRightString(tag, "&");
      writeStartTag(leftTag);
      writeText(rightTag, text);
      writeEndTag(leftTag);
    }
    else {
      writeText(tag, text);
    }
  }

  public void writeElement(String tag, String text, boolean withAttribute) throws IOException {
    String attribute;
    if ( withAttribute && (attribute = getAttribute(text))!= null ) {
      if (tag.indexOf("&") > 0) {
        String leftTag = getLeftString(tag, "&");
        String rightTag = getRightString(tag, "&");
        writeStartTag(leftTag, attribute);
        text = text.substring(0, text.indexOf(attribute)).trim();
        writeText(rightTag, text);
        writeEndTag(leftTag);
      }
      else {
        text = text.substring(0, text.indexOf(attribute)).trim();
        writeText(tag, text, attribute);
      }
    }
    else
      writeElement(tag, text);
  }

  public String writeElement(String tag, String text, TagStack tagStack) throws IOException {
    if (tag.indexOf("&") > 0) {
      String leftTag = getLeftString(tag, "&");
      String rightTag = getRightString(tag, "&");
      tagStack.push(leftTag);
      writeStartTag(leftTag);
      writeText(rightTag, text);

      return leftTag;  // context
    }
    else
      writeElement(tag, text);

    return null;
  }

  public String writeElement(String tag, String text, TagStack tagStack, boolean withAttribute) throws IOException {
    String attribute = null;
    if ( withAttribute && (attribute = getAttribute(text))!= null ) {
      if (tag.indexOf("&") > 0) {
        String leftTag = getLeftString(tag, "&");
        String rightTag = getRightString(tag, "&");
        tagStack.push(leftTag);
        writeStartTag(leftTag, attribute);
        text = text.substring(0, text.indexOf(attribute)).trim();
        writeText(rightTag, text);

        return leftTag;    // context
      }
      return null;
    }
    else
      return writeElement(tag, text, tagStack);
  }

  public void close() throws IOException {
    if (title != null) xmlFile.write("</"+title+">");
    xmlFile.close();
  }

  public static String getAttribute(String str) {
    String attribute = null;
    if ( str.endsWith(")") && !str.startsWith("(") ) {
      int beginIndex = str.indexOf("(");
      int endIndex = str.indexOf(")");
      if ( beginIndex >= 0 && beginIndex < endIndex ) {
        attribute = str.substring(beginIndex, endIndex+1);
      }
      return attribute;
    }
    return null;
  }

  public static String getLeftString(String str, String delim) {
    if (str.indexOf(delim) > 0) {
      int beginIndex = 0;
      int endIndex = str.indexOf(delim);
      if (beginIndex <= endIndex) {
        String leftStr = str.substring(beginIndex, endIndex).trim();
        return leftStr;
      }
    }
    return null;
  }

  public static String getRightString(String str, String delim) {
    if (str.indexOf(delim) >= 0) {
      int beginIndex = str.indexOf(delim)+delim.length();
      int endIndex = str.length();
      if (beginIndex <= endIndex) {
        String rightStr = str.substring(beginIndex, endIndex).trim();
        return rightStr;
      }
    }
    return null;
  }
}
